@@ -13,8 +13,11 @@
 #include "testdir.h"
 
 #include <KActionCollection>
+#include <KConfig>
+#include <KConfigGui>
 
 #include <QAccessible>
+#include <QFileSystemWatcher>
 #include <QScopedPointer>
 #include <QSignalSpy>
 #include <QStandardPaths>
@@ -43,6 +46,7 @@ private Q_SLOTS:
     void testGoActions();
     void testOpenFiles();
     void testAccessibilityAncestorTree();
+    void testAutoSaveSession();
     void cleanupTestCase();
 
 private:
@@ -208,7 +212,8 @@ void DolphinMainWindowTest::testUpdateWindowTitleAfterChangingSplitView()
 // Test case for bug #397910
 void DolphinMainWindowTest::testOpenInNewTabTitle()
 {
-    m_mainWindow->openDirectories({QUrl::fromLocalFile(QDir::homePath())}, false);
+    const QUrl homePathUrl{QUrl::fromLocalFile(QDir::homePath())};
+    m_mainWindow->openDirectories({homePathUrl}, false);
     m_mainWindow->show();
     QVERIFY(QTest::qWaitForWindowExposed(m_mainWindow.data()));
     QVERIFY(m_mainWindow->isVisible());
@@ -216,13 +221,14 @@ void DolphinMainWindowTest::testOpenInNewTabTitle()
     auto tabWidget = m_mainWindow->findChild<DolphinTabWidget *>("tabWidget");
     QVERIFY(tabWidget);
 
-    tabWidget->openNewTab(QUrl::fromLocalFile(QDir::tempPath()));
+    const QUrl tempPathUrl{QUrl::fromLocalFile(QDir::tempPath())};
+    tabWidget->openNewTab(tempPathUrl);
     QCOMPARE(tabWidget->count(), 2);
     QVERIFY(tabWidget->tabText(0) != tabWidget->tabText(1));
-    if (!tabWidget->tabIcon(0).isNull() && !tabWidget->tabIcon(1).isNull()) {
-        QCOMPARE(QStringLiteral("inode-directory"), tabWidget->tabIcon(0).name());
-        QCOMPARE(QStringLiteral("inode-directory"), tabWidget->tabIcon(1).name());
-    }
+
+    QVERIFY2(!tabWidget->tabIcon(0).isNull() && !tabWidget->tabIcon(1).isNull(), "Tabs are supposed to have icons.");
+    QCOMPARE(KIO::iconNameForUrl(homePathUrl), tabWidget->tabIcon(0).name());
+    QCOMPARE(KIO::iconNameForUrl(tempPathUrl), tabWidget->tabIcon(1).name());
 }
 
 void DolphinMainWindowTest::testNewFileMenuEnabled_data()
@@ -535,32 +541,86 @@ void DolphinMainWindowTest::testAccessibilityAncestorTree()
     QVERIFY(QTest::qWaitForWindowExposed(m_mainWindow.data()));
     QVERIFY(m_mainWindow->isVisible());
 
-    std::set<const QObject *> testedObjects; // Makes sure we stop testing if we arrive at an item that was already tested.
     QAccessibleInterface *accessibleInterfaceOfMainWindow = QAccessible::queryAccessibleInterface(m_mainWindow.get());
     Q_CHECK_PTR(accessibleInterfaceOfMainWindow);
 
-    // We will do accessibility checks for every object that gets focus. Focus will be changed using the Tab key.
-    while (qApp->focusObject() && !testedObjects.count(qApp->focusObject())) {
-        const auto currentlyFocusedObject = qApp->focusObject();
-        QAccessibleInterface *accessibleInterface = QAccessible::queryAccessibleInterface(currentlyFocusedObject);
-
-        // The accessibleInterfaces of focused objects might themselves have children.
-        // We go down that hierarchy as far as possible and then test the ancestor tree from there.
-        while (accessibleInterface->childCount() > 0) {
-            accessibleInterface = accessibleInterface->child(0);
-        }
-        while (accessibleInterface != accessibleInterfaceOfMainWindow) {
-            QVERIFY2(accessibleInterface,
-                     qPrintable(QString("%1's accessibleInterface or one of its accessible children doesn't have the main window as an ancestor.")
-                                    .arg(currentlyFocusedObject->metaObject()->className())));
-            accessibleInterface = accessibleInterface->parent();
+    // We will test the accessibility of objects traversing forwards and backwards.
+    int testedObjectsSizeAfterTraversingForwards = 0;
+    for (int i = 0; i < 2; i++) {
+        std::tuple<Qt::Key, Qt::KeyboardModifier> focusChainTraversalKeyCombination = {Qt::Key::Key_Tab, Qt::NoModifier};
+        if (i) {
+            focusChainTraversalKeyCombination = {Qt::Key::Key_Tab, Qt::ShiftModifier};
         }
 
-        testedObjects.insert(currentlyFocusedObject); // Add it to testedObjects so we won't test it again later.
-        QTest::keyClick(m_mainWindow.get(), Qt::Key::Key_Tab, Qt::ShiftModifier); // ShiftModifier because the Tab cycle is currently broken going forward.
+        // We will do accessibility checks for every object that gets focus. Focus will be changed using the focusChainTraversalKeyCombination.
+        std::set<const QObject *> testedObjects; // Makes sure we stop testing when we arrive at an item that was already tested.
+        while (qApp->focusObject() && !testedObjects.count(qApp->focusObject())) {
+            const auto currentlyFocusedObject = qApp->focusObject();
+
+            QAccessibleInterface *accessibleInterface = QAccessible::queryAccessibleInterface(currentlyFocusedObject);
+            // The accessibleInterfaces of focused objects might themselves have children.
+            // We go down that hierarchy as far as possible and then test the ancestor tree from there.
+            while (accessibleInterface->childCount() > 0) {
+                accessibleInterface = accessibleInterface->child(0);
+            }
+            while (accessibleInterface != accessibleInterfaceOfMainWindow) {
+                QVERIFY2(accessibleInterface,
+                         qPrintable(QString("%1's accessibleInterface or one of its accessible children doesn't have the main window as an ancestor.")
+                                        .arg(currentlyFocusedObject->metaObject()->className())));
+                accessibleInterface = accessibleInterface->parent();
+            }
+
+            testedObjects.insert(currentlyFocusedObject); // Add it to testedObjects so we won't test it again later.
+            QTest::keyClick(m_mainWindow.get(), std::get<0>(focusChainTraversalKeyCombination), std::get<1>(focusChainTraversalKeyCombination));
+            QVERIFY2(currentlyFocusedObject != qApp->focusObject(),
+                     "The focus chain is broken. The focused object should have changed after pressing the focusChainTraversalKeyCombination.");
+        }
+
+        if (i == 0) {
+            testedObjectsSizeAfterTraversingForwards = testedObjects.size();
+        } else {
+            QCOMPARE(testedObjects.size(), testedObjectsSizeAfterTraversingForwards); // The size after traversing backwards is different than
+                                                                                      // after going forwards which is probably not intended.
+        }
     }
 }
 
+void DolphinMainWindowTest::testAutoSaveSession()
+{
+    m_mainWindow->openDirectories({QUrl::fromLocalFile(QDir::homePath())}, false);
+    m_mainWindow->show();
+    QVERIFY(QTest::qWaitForWindowExposed(m_mainWindow.data()));
+    QVERIFY(m_mainWindow->isVisible());
+
+    // Create config file
+    KConfigGui::setSessionConfig(QStringLiteral("dolphin"), QStringLiteral("dolphin"));
+    KConfig *config = KConfigGui::sessionConfig();
+    m_mainWindow->saveGlobalProperties(config);
+    m_mainWindow->savePropertiesInternal(config, 1);
+    config->sync();
+
+    // Setup watcher for config file changes
+    const QString configFileName = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + "/" + KConfigGui::sessionConfig()->name();
+    QFileSystemWatcher *configWatcher = new QFileSystemWatcher({configFileName}, this);
+    QSignalSpy spySessionSaved(configWatcher, &QFileSystemWatcher::fileChanged);
+
+    // Enable session autosave.
+    m_mainWindow->setSessionAutoSaveEnabled(true);
+    m_mainWindow->m_sessionSaveTimer->setInterval(200); // Lower the interval to speed up the testing
+
+    // Open a new tab
+    auto tabWidget = m_mainWindow->findChild<DolphinTabWidget *>("tabWidget");
+    QVERIFY(tabWidget);
+    tabWidget->openNewActivatedTab(QUrl::fromLocalFile(QDir::tempPath()));
+    QCOMPARE(tabWidget->count(), 2);
+
+    // Wait till a session save occurs
+    QVERIFY(spySessionSaved.wait(60000));
+
+    // Disable session autosave.
+    m_mainWindow->setSessionAutoSaveEnabled(false);
+}
+
 void DolphinMainWindowTest::cleanupTestCase()
 {
     m_mainWindow->showNormal();
