@@ -64,7 +64,7 @@ void Trash::empty(QWidget *window)
 bool Trash::isEmpty()
 {
     KConfig trashConfig(QStringLiteral("trashrc"), KConfig::SimpleConfig);
-    return (trashConfig.group("Status").readEntry("Empty", true));
+    return (trashConfig.group(QStringLiteral("Status")).readEntry("Empty", true));
 }
 
 #include "moc_dolphintrash.cpp"
